"""
Readline completer
"""

# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import types
from Queue import Queue

try:
	from xreadline import get_line_buffer,update,parse_and_bind
	xrd = True
except:
	from readline import get_line_buffer,parse_and_bind
	xrd = False
	def update():
		print

class completer:

	__cmps = {}
	__cmps_temp = []
	parse = None
	queue = None

	def __init__(self,parse):
		parse_and_bind('"\C-o": kill-whole-line')
		parse_and_bind('"\C-q": "\C-oquit\n"')
		self.parse = parse
		self.queue = Queue()
		self.namespaces = [".cli","parameter"]

	def display_type(self,text,key):
		result = ""
		for i in text:
			if i != "%noprint%" and self.__cmps[key].has_key(i):
				if key == "parameter":
					result +=  "%-16s - %s: <%s>\n" % (i,self.__cmps[key][i]["description"],str(self.__cmps[key][i]["constraint"]))
				else:
					result +=  "%-16s - %s\n" % (i,self.__cmps[key][i]["description"])
		if result:
			print "\n\t* %s:" % (key)
			print result,
	

	def display(self,text,key):
		print "\n"
		for i in self.namespaces:
			self.display_type(text,i)
		print


	def complete(self,text,state):

##		get current prefix string
		prefix = get_line_buffer().rstrip(text).strip()
		stack = ""
		for i in prefix.split():
			stack += "." + i
		if len(stack):
			stack = stack[1:]
		if prefix:
			prefix = "prefix " + prefix


##		populate results array
		if not len(self.__cmps_temp) and not state:

			try:
				self.parse("list " + prefix, "listing", [])
				# parse listing
				from xml.dom.minidom import parseString
				a = self.queue.get()
				if a[:4] == "data":
					b = eval(a[5:])
					a = b[2]
				dom = parseString(a)
				tmp = {}

				for i in dom.documentElement.childNodes:
					if i.localName == "parameters":
						key = "parameter"
					else:
						key = i.getAttribute("ns")
					tmp[key] = {}
					for k in i.childNodes:
						desc = ""
						desc += str(k.getElementsByTagName("desc")[0].firstChild.data)
						cst = k.getAttribute("type")
						if cst == "enum":
							cst += " ["
							for l in k.getElementsByTagName("v"):
								cst += str(l.firstChild.data) + ","
							cst = cst[:-1] + "]"
						tmp[key][k.getAttribute("name")] = {
							"description": desc,
							"constraint": cst,
						}
				tmp = ["list",tmp]

			except Exception,e:
				print "\n\nError while requesting the core: %s\n" % (e)
				update()
				return 0

			if type(tmp) != types.ListType:
				print "\n\nThe core returned a garbage: %s\n" % (tmp)
				update()
				return 0

			if tmp[0] == "error":
				## that is an error: display it and exit
				print "\n\n%s\n" % (tmp[1])
				update()
				return 0

			elif tmp[0] == "parameter":
				## parameter type and description: print it and exit
				print "\n\n%-16s- (%s) %s\n" % ("<" + str(tmp[2]["constraint"]) + ">",tmp[1],tmp[2]["description"])
				update()
				return 0

			elif tmp[0] == "list":
				## full list
				self.__cmps = tmp[1]
				if not self.__cmps.has_key("parameter"):
					self.__cmps["parameter"] = {}
				self.__cmps[".cli"] = {}
			
			else:
				print "\n\nThe core returned a garbage of type %s\n" % (tmp[0])
				update()
				return 0

			if not prefix:
				self.__cmps[".cli"][".email"] = {
					"description": "Enter info for e-mail forwarding"
				}
				self.__cmps[".cli"][".cli"] = {
					"description": "Start /bin/sh session"
				}
				self.__cmps[".cli"][".source"] = {
					"description": "Source script file"
				}

			for (l,m) in self.__cmps.items():
				for (i,k) in m.items():
					if i.startswith(text):
						self.__cmps_temp.append(i)
					if not l in self.namespaces:
						self.namespaces.append(l)

			if len(self.__cmps_temp) and (not text) and (xrd):
				self.__cmps_temp.append("%noprint%")
		
##		return next value in cmps_temp
		if len(self.__cmps_temp):
			return self.__cmps_temp.pop(0)

##		return non-string at last
		return 0
