#!/usr/bin/env python
"""
Connexion CLI: main module
"""

# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from getopt import getopt
import sys
import os
import tempfile

# commented out due to ALT specific, till it'll be solved
#
# from setproctitle import setproctitle
#
# setproctitle("connexion cli")

rc = ".connexionrc"
socket = "/".join((tempfile.gettempdir(),"socket"))
prompt = True
input = True
remove = False
async = False
daemon = False
# we must provide at least one item in the dict,
# otherwise it will be omitted in request :(
trusted = {
	"pid": 0
	}
execute = []

def usage():
	print '''
Options:
	-a		start shell session in async mode (speeds up scripts)
	-b		do not poll stdin (for daemon mode calls)
	-d <pass>	start embedded debugger with pass (needs rpdb2)
	-e "<command;>"	execute commands
	-h		this help
	-n		do no print prompt (e.g. when process commands from stdin)
	-r <file>	rc file to run
	-s <file>	socket to use
	-u <user:group>	user and group names (only with trusted socket)
	-x <file>	script to run, like -r, but exit after script execution
	-X <file>	as -x, but remove the script after execution

Resource files:
	Resource file can contain any defined commands. Strings,
	beginning from # are treated as comments that must be
	ignored

Autocompletion:
	Connexion CLI uses standard GNU readline library. To get
	completions, type "TAB" or "?"
'''

try:
	(opts,left) = getopt(sys.argv[1:],"abd:he:nr:s:u:x:X:")
except Exception, e:
	print e
	usage()
	sys.exit(255)

for (i,k) in opts:
	if i == "-a":
		async = True
	elif i == "-b":
		daemon = True
	elif i == "-h":
		usage()
		sys.exit(0)
	elif i == "-e":
		execute = k
		execute = execute.split(";")
		input = False
	elif i == "-n":
		prompt = False
	elif i == "-r":
		rc = k
	elif i == "-s":
		socket = k
	elif i == "-u":
		k = k.split(":")
		trusted = {
			"user": k[0],
			"group": k[1],
			"pid": os.getpid(),
		}
	elif i == "-x" or i == "-X":
		rc = k
		input = False
	elif i == "-X":
		remove = True
	elif i == "-d":
		try:
			import rpdb2
			print "starting rpdb2"
			rpdb2.start_embedded_debugger(k)
		except:
			print "no rpdb2 found, continuing w/o debugging"

from shell import shell as _shell
shell = _shell(socket,trusted)
shell.async = async
shell.daemon = daemon
shell.process_rc(rc,remove)

for i in execute:
	shell.exec_line(i)

retcode = 0
if input:
	retcode = shell.input_loop(prompt)

shell.terminate()
sys.exit(retcode)
