"""
Shell implementation
"""

# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import urllib
import types
import locale

backup = urllib.splithost

def xsplithost(uri):
	return (uri,0)

urllib.splithost = xsplithost

import socket
import httplib

def xconnect(self):
    """Connect to the host and port specified in __init__."""
    msg = "getaddrinfo returns an empty list"
    try:
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM, 0)
        if self.debuglevel > 0:
            print "connect: (%s, %s)" % (self.host, self.port)
        self.sock.connect(self.host)
    except socket.error, msg:
        if self.debuglevel > 0:
            print 'connect fail:', (self.host, self.port)
        if self.sock:
            self.sock.close()
        self.sock = None
    if not self.sock:
        raise socket.error, msg


httplib.HTTPConnection.connect = xconnect

import xmlrpclib
import sys
import os
from select import poll, POLLIN, POLLHUP
import signal

from base64 import b64decode
from email import MIMEMultipart, message_from_string
import smtplib

import re
from copy import copy

from completer import completer

from threading import Timer

try:
	from xreadline import parse_and_bind, set_completer, set_display_hook, update, get_completer_delims, set_completer_delims
except:
	from readline import parse_and_bind, set_completer, get_completer_delims, set_completer_delims
	def update():
		print

	def set_display_hook(*arg):
		pass

class shell:

	completer = None
	server = None
	oid = None
	prompt = None
	encoding = None
	email = None
	smtp = None
	source = None
	prefs = None
	buffer = None
	timer = None
	ready = None
	async = None
	daemon = None
	root = None

	trusted = None
	cmap = None
	
	def __init__(self,socket,trusted):
		self.server = xmlrpclib.Server('http:' + socket)
		self.trusted = trusted
		try:
			self.oid = self.server.start(self.trusted)
			self.root = self.server.init(self.trusted)["prompt"]


		except Exception,e:
			print "Connection error:",e
			sys.exit(1)
		self.prompt = "%s > " % (self.root)
		self.completer = completer(self.queue)
		self.cmap = {
			"listing": self.completer.queue,
		}
		self.encoding = locale.getdefaultlocale()[1]
		self.email = ""
		self.smtp  = ""
		self.source = ""
		self.prefs = ["",]

		self.buffer = {}
		self.timer = None
		self.async = False
		self.daemon = False

		if not self.encoding:
			self.encoding = "ascii"

	def process_rc(self,file,remove):
		try:
			self.source = open(file,'r')
		except:
			return

		while True:
			try:
				i = self.getline("")
			except:
				if remove:
					os.unlink(file)
				del self.source
				self.source = ""
				return

			try:
				if re.match(i,"^[\ \t]*#.*$"):
					continue
			except:
				continue

			self.exec_line(i)

	def process_stdin(self):
		x = poll()
		if not self.daemon:
			x.register(sys.stdin, POLLIN)
		input = ""

		if self.email and self.smtp:
			msg = MIMEMultipart.MIMEMultipart()
			msg["From"] = "cli@connexion.radlinux.org"
			msg["To"] = self.email
			msg["Subject"] = "cli@connexion report"

		interval = 50

		while True:
			try:
				l = x.poll(interval)
				if self.async:
					interval = 0
				else:
					interval = 500
##				The fuck is in that the poll does not honour our event mask in poll.register(),
##				and returns also POLLHUP. But we want only POLLIN, otherwise we'll hang
##				on stdin.
## <ugly-hack>
				for i in l:
					if i[1] != POLLIN:
						l.remove(i)
## </ugly-hack>

				if not len(l):
##					Another bug: if we catch a signal while waiting a server
##					response, then we _loose_ the response.
##
##					Or we can ignore SIGINT while waiting a response, but this
##					way leads to responsibility issues.
## <ugly-hack>
					if self.async:
						result = self.server.input(self.trusted,self.oid,input)
					else:
						sig = signal.signal(signal.SIGINT,signal.SIG_IGN)
						result = self.server.input(self.trusted,self.oid,input)
						signal.signal(signal.SIGINT,sig)
## </ugly-hack>

					input = ""
					queue = result["queue"]
					for i in result["data"]:
						if i.has_key("data"):
							try:
								message = message_from_string(i["data"])
								if self.email and self.smtp:
									msg.attach(message)

								text = b64decode(message.get_payload())

								if self.cmap.has_key(i["cookie"]):
									self.cmap[i["cookie"]].put(text)
								else:

									xtype = message.get_content_type()
									if xtype in ("text/plain","text/xml"):
										charset = message.get_content_charset()
										if not charset:
											charset = 'utf-8'
										text = text.decode(charset,'xmlcharrefreplace')
										text = text.encode(self.encoding,'xmlcharrefreplace')
										print text,
									else:
										print "got message of type %s" % (xtype)

									sys.stdout.flush()
							except:
								pass

					if result.has_key("prompt"):
						self.prompt = result["prompt"] + " > "

					if queue == 0:
						sys.stdout.flush()
						if self.async:
							update()
						break
				
				for (fd,event) in l:
					a = os.read(fd,1)
					if (event & POLLIN) and (fd == sys.stdin.fileno()):
						input += a
			
			except KeyboardInterrupt:
				self.server.interrupt(self.trusted,self.oid,'c')

			except Exception, e:
				import traceback
				print e
				traceback.print_exc()

		if self.email and self.smtp and msg.is_multipart():
			print "\nsending email... ",
			try:
				s = smtplib.SMTP(self.smtp)
				s.sendmail("cli@connexion.radlinux.org",[self.email,],msg.as_string())
				s.quit()
				del s
				print "done"
			except Exception,e:
				print "exception: %s" % (e)

	def flush(self):
		if self.timer:
			self.timer.cancel()
			del self.timer
		self.timer = False

		for (i,k) in self.buffer.items():
			result = self.server.parse(self.trusted,i,self.oid,k)
			del self.buffer[i]

			if result.has_key("error"):
				print "%s" % (result["error"])

			if result.has_key("prompt"):
				self.prompt = result["prompt"] + " > "

		self.process_stdin()

	def queue(self,text,cookie,lines):

		if self.timer:
			self.timer.cancel()
			del self.timer

		if self.async:
			self.timer = Timer(0.2,self.flush)

		if not self.buffer.has_key(cookie):
			self.buffer[cookie] = []
		self.buffer[cookie].append([text,lines])

		if self.async:
			self.timer.start()
		else:
			self.flush()

	def exec_line(self,text,analyze=True,cookie="none"):
##		look if we have multiline input
		pref = re.sub("[^\ \t].*","",text)

		if pref < self.prefs[-1:][0] and analyze:
			backup = copy(self.prefs)
			while self.prefs:
				if pref == self.prefs[-1:][0]:
					for i in xrange(len(backup) - len(self.prefs)):
						self.exec_line("pop",False)
					backup = False
					break
				self.prefs.pop()
			if backup:
				print "Warning: illegal indent, ignoring"
				del self.prefs
				self.prefs = backup

		elif pref > self.prefs[-1:][0]:
			self.prefs.append(pref)

		text = text.strip()
		lines = []
		if text[len(text) - 2:] == "<<":
			text = text[:-2]
			while True:
				line = self.getline("... ")
				if line == ".":
					break
				lines.append(line)
		
		if text:
			if text == "quit":
				print "got to quit"
				raise SystemExit()
			if text[:7] == ".source":
				self.process_rc(text[8:], False)
				return
			if text == ".cli":
				os.system("/bin/sh")
				return
			if text == ".email":
				self.smtp  = self.getline("ipaddr or name of smtp server: ")
				self.email = self.getline("email address to send output to: ")
				return
			if text == "shutdown":
				self.server.parse(self.trusted,cookie,self.oid,[[text,[]],])
				sys.exit(0)

			self.queue(text,cookie,lines)
			

	def get_prompt(self,prompt = True):
		if prompt:
			return self.prompt
		else:
			return ""

	def getline(self,prompt):
		if self.source:
			a = self.source.readline()
			if len(a) == 0:
				raise Exception("EOF reached")
			return a.rstrip('\n')
		else:
			return raw_input(prompt)

	def input_loop(self,prompt = True):
##		FIXME: setup history saving

##		setup readline hooks
		parse_and_bind("tab: complete")
		parse_and_bind("?: complete")
		set_completer_delims(" :;")
		set_completer(self.completer.complete)
		set_display_hook(self.completer.display)

		while True:
			try:
				text = raw_input(self.get_prompt(prompt)).decode(self.encoding).encode('utf-8')
				self.exec_line(text)

			except KeyboardInterrupt:
				print

			except EOFError:
## <ugly-hack>
## FIXME:
				if self.prompt == "%s > " % (self.root):
					print
					break
## </ugly-hack>
				if not self.source:
					print "pop"
				else:
					print self.source
				if len(self.prefs) > 1:
					self.prefs.pop()
				self.exec_line("pop",False)

			except SystemExit:
				break

			except:
				pass
		
		return 0

	def terminate(self):
		#if self.timer:
		#	self.timer.finished.wait()
		if self.async:
			self.flush()

		self.server.terminate(self.trusted,self.oid)


